/******************************************************************************
 *
 * FILE: can.c
 *
 * Description:
 *    CAN communication
 *
 * History:
 *    17.11.2017   PN   Initial version
 *    13.02.2018   MF   Reuse for SPC574K72
 *
 ******************************************************************************
 * Copyright (C) 2015-2017 HighTec EDV-Systeme GmbH.  All rights reserved.
 *
 * This is proprietary software. Any use of the software requires a written
 * license agreement with HighTec EDV-Systeme GmbH. Please contact for
 * detailed license information: info@hightec-rt.com.
 *****************************************************************************/

/* BSP support package */
#include "bsp.h"
/* CAN communication header */
#include "can.h"

/* ============================================================================
 *            GLOBAL VARIABLES
 * ==========================================================================*/

CAN_MESSAGE_OBJECT_t g_can_RxData;
CAN_MESSAGE_OBJECT_t g_can_TxData = \
		{.ID = 0x10, .XTD = 0, .BRS = 0, .DLC = 8, .EDL = 1,     \
		 .Data[0] = 0, .Data[1] = 1, .Data[2] = 2, .Data[3] = 3, \
         .Data[4] = 4, .Data[5] = 5, .Data[6] = 6, .Data[7] = 7};

/* ============================================================================
 *            ISR FUNCTIONS
 * ==========================================================================*/

/* ----------------------------------------------------------------------------
 * FUNCTION: CanIsrHandler()
 *    RX FIFO 0 ISR Handler
 * --------------------------------------------------------------------------*/

void CanIsrHandler(void)
{
	if (CAN_SUB_0_M_CAN_1.IR.B.RF0N == 1)	// Received MSG
	{
		/* ID */
		g_can_RxData.ID = ((CAN_RX_FIFO_RAM.B.ID >> 1) & 0x7FF);
		g_can_RxData.XTD = 0;

		 /* Length */
		g_can_RxData.DLC = (CAN_RX_FIFO_RAM.B.DLC  & 0x0F);

		/* DATA */
		g_can_RxData.Data[0] = CAN_RX_FIFO_RAM.B.DB0;
		g_can_RxData.Data[1] = CAN_RX_FIFO_RAM.B.DB1;
		g_can_RxData.Data[2] = CAN_RX_FIFO_RAM.B.DB2;
		g_can_RxData.Data[3] = CAN_RX_FIFO_RAM.B.DB3;
		g_can_RxData.Data[4] = CAN_RX_FIFO_RAM.B.DB4;
		g_can_RxData.Data[5] = CAN_RX_FIFO_RAM.B.DB5;
		g_can_RxData.Data[6] = CAN_RX_FIFO_RAM.B.DB6;
		g_can_RxData.Data[7] = CAN_RX_FIFO_RAM.B.DB7;

		/* acknowledge register */
		CAN_SUB_0_M_CAN_1.RXF0A.B.F0AI = 0x0;
	}
	/* Clear INT flag */
	CAN_SUB_0_M_CAN_1.IR.B.RF0N = 0x1;
}

/* ============================================================================
 *            CAN example API
 * ==========================================================================*/
/* ----------------------------------------------------------------------------
 * Name:        can_Init
 * Parameters:
 *              none
 * Returns:
 *              none
 * Description: Initialization of M_CAN_1.
 *
 * --------------------------------------------------------------------------*/
void can_Init(void)
{ 
    uint32_t* p_memory;

    /* Clear Shared CAN Message RAM */
    p_memory = (uint32_t*)(CAN_SHARED_RAM_ADD_START);
    while(p_memory < (uint32_t*)(CAN_SHARED_RAM_ADD_END))
    {
        *p_memory = 0;
        p_memory++;
    }

    /* Setting the INIT bit high to start the initialization */
    CAN_SUB_0_M_CAN_1.CCCR.R = 1;
    while ((CAN_SUB_0_M_CAN_1.CCCR.R & 0x3) != 0x3)
    {
    	CAN_SUB_0_M_CAN_1.CCCR.R = (uint32_t)0x3;
    }

    /* Configuring the time stamp and timeout registers */
    CAN_SUB_0_M_CAN_1.TSCV.R = 0; /* Time stamp is not used */
    CAN_SUB_0_M_CAN_1.TOCC.R = 0; /* Timeout counter is not used */

    /* Configuring the interrupt registers */
    CAN_SUB_0_M_CAN_1.IE.B.RF0NE =  1; /* Rx FIFO 0 New Message Interrupt Enable */
    CAN_SUB_0_M_CAN_1.ILS.B.RF0NL = 0;	/* RxFIFO 0 new message is assigned to interrupt
                               line 0 */
    CAN_SUB_0_M_CAN_1.ILE.B.EINT0 = 1; /* Enable interrupt from line 0 */

    CAN_SUB_0_M_CAN_1.GFC.R = 0;            /* The standard and Extended frames are
                                    accepted in Rx FIFO_0*/
    CAN_SUB_0_M_CAN_1.XIDAM.R = 0x1FFFFFFF; /* Extended ID mask, AND mask is not active */

    /* Writing the start addresses of all the M_CAN Buffers */
    CAN_SUB_0_M_CAN_1.RXF0C.R = CAN_RX_FIFO_OFFSET;
    CAN_SUB_0_M_CAN_1.RXF0C.B.F0S = 1;      /* One Rx FIFO 0 element */
    CAN_SUB_0_M_CAN_1.RXF0C.B.F0WM = 0;     /* Watermark interrupt disabled */
    CAN_SUB_0_M_CAN_1.RXF0C.B.F0OM = 1;     /* FIFO 0 overwrite mode*/

    CAN_SUB_0_M_CAN_1.RXF1C.B.F1S = 0;      /* NO Rx FIFO 1 element */

    CAN_SUB_0_M_CAN_1.SIDFC.R = 0x0;        /* No standard Message ID filter */
    CAN_SUB_0_M_CAN_1.XIDFC.R = 0x0;        /* No extended Message ID filter */

    CAN_SUB_0_M_CAN_1.TXBC.R = CAN_TX_FIFO_OFFSET;
    CAN_SUB_0_M_CAN_1.TXBC.B.NDTB = 1;      /* One Dedicated Tx Buffers */
    CAN_SUB_0_M_CAN_1.TXBC.B.TFQS = 0;      /* No Tx Buffers used for Tx FIFO*/
    CAN_SUB_0_M_CAN_1.TXBC.B.TFQM = 0;      /* Tx FIFO operation */

    CAN_SUB_0_M_CAN_1.TXEFC.R = 0x0;        /* Tx events disabled */

    /* Writing to the RAM for filter configuration */
    /* No filter configuration */

    CAN_SUB_0_M_CAN_1.SIDFC.R = CAN_STD_FILTER_OFFSET;
    CAN_SUB_0_M_CAN_1.SIDFC.B.LSS   = 1;

    /* M_CAN Baud Rate configuration*/
    CAN_SUB_0_M_CAN_1.NBTP.R = CAN_BTP_CFG;

    CCCU_0.CCFG.R = 0x444; /* Bypass Clock Calibration on CAN unit */

    /* End of CAN initialization*/
#if(CAN_LOOP_BACK_TEST)
    /* External Loop Back Test mode */
    CAN_SUB_0_M_CAN_1.CCCR.R = 0x83;
    CAN_SUB_0_M_CAN_1.TEST.B.LBCK = 1;
    CAN_SUB_0_M_CAN_1.CCCR.R = 0x80;
#else
    CAN_SUB_0_M_CAN_1.CCCR.B.INIT = 0;
#endif

    while(CAN_SUB_0_M_CAN_1.CCCR.B.INIT == 1);

    CAN_SUB_0_M_CAN_1.CCCR.B.ASM = 0;       /*  Normal CAN operation */

    /* Register interrupt handler */
    bsp_isr_RegisterHandler(UC_CAN_ISR_INT0, 4, CanIsrHandler);

    /* Configure M_CAN_1 RX and TX pins  */
	bsp_pin_EnableOutput(CAN_PIN_PA10, CAN_PIN_SSS_TX); /* M_CAN_1_TX, PA10 */
	bsp_pin_EnableInput(CAN_PIN_PA11);                  /* M_CAN_1_RX, PA11 */
	/* M_CAN1_RX functionality */
	SIUL2.MSCR_MUX[CAN_PIN_MUX_PA11].B.SSS = CAN_PIN_SSS_RX;
}

/* ----------------------------------------------------------------------------
 * Name:        can_SendData
 * Parameters:
 *              none
 * Returns:
 *              none
 * Description:
 *              Function for frame send
 * --------------------------------------------------------------------------*/
void can_SendData(void)
{
		/* copy data to TX FIFO */
		CAN_TX_FIFO_RAM.R.T2 = \
				(uint32_t)((((uint32_t)(g_can_TxData.Data[3]))<<24)| \
				 		   (((uint32_t)(g_can_TxData.Data[2]))<<16)| \
						   (((uint32_t)(g_can_TxData.Data[1]))<<8) | \
								        g_can_TxData.Data[0]);

		CAN_TX_FIFO_RAM.R.T3 = \
				(uint32_t)((((uint32_t)(g_can_TxData.Data[7]))<<24)| \
                           (((uint32_t)(g_can_TxData.Data[6]))<<16)| \
                           (((uint32_t)(g_can_TxData.Data[5]))<<8) | \
                                        g_can_TxData.Data[4]);

		CAN_TX_FIFO_RAM.B.DLC = g_can_TxData.DLC;

		/* 11-bit standard identifier */
		CAN_TX_FIFO_RAM.B.BRS = g_can_TxData.BRS;
		CAN_TX_FIFO_RAM.B.EDL = g_can_TxData.EDL;
		CAN_TX_FIFO_RAM.B.ID  = g_can_TxData.ID << 18;
		CAN_TX_FIFO_RAM.B.XTD = g_can_TxData.XTD;

		/* Increments data[0] for next transmission */
		g_can_TxData.Data[0] += 1;

		/* Request to send prepared messages */
		CAN_SUB_0_M_CAN_1.TXBAR.B.AR0 = 0x1;
}

// End of file
